@@ -9,30 +9,32 @@ updateDimRedSelInput <-
     }
 #' @importFrom utils adist
 updateDimRedSelInputPair <-
-    function(session, input, dataSource) {
+    function(session, input, dataSource, ABcolumn='') {
+        idx <- paste0("GeneExprdrX", ABcolumn)
+        idy <- paste0("GeneExprdrY", ABcolumn)
         updateDimRedSelInput(
             session,
-            "GeneExprdrX",
+            idx,
             "X-axis:",
             dataSource()$sc1conf,
             dataSource()$sc1def$dimred[1]
         )
         updateDimRedSelInput(
             session,
-            "GeneExprdrY",
+            idy,
             "Y-axis:",
             dataSource()$sc1conf,
             dataSource()$sc1def$dimred[2]
         )
-        observeEvent(input$GeneExprdrX, {
+        observeEvent(input[[idx]], {
             try({
                 conf <- dataSource()$sc1conf
                 choices <- conf[conf$dimred == TRUE]$UI
-                choices <- choices[choices!=input$GeneExprdrX]
-                dist <- adist(input$GeneExprdrX, choices)
+                choices <- choices[choices!=input[[idx]]]
+                dist <- adist(input[[idx]], choices)
                 updateDimRedSelInput(
                     session, 
-                    "GeneExprdrY",
+                    idy,
                     "Y-axis:",
                     dataSource()$sc1conf,
                     choices[which.min(dist)])
@@ -305,7 +307,8 @@ updateGeneExprDotPlotUI <-
                         .globals$figHeight, height,
                     input[[paste0("GeneExproup.h", postfix)]]*72),
                 dblclick = NS0(id, 'GeneExproup.dbl', postfix),
-                click = NS0(id, 'GeneExproup.clk', postfix),
+                click = clickOpts(NS0(id, 'GeneExproup.clk', postfix),
+                                  clip = FALSE),
                 brush = brushOpts(NS0(id, 'GeneExproup.brush', postfix),
                                   resetOnNew = TRUE))
         })
@@ -561,10 +564,6 @@ updateGeneExprDotPlotUI <-
         }else{## not info
             ## zoom in for ATAC
             ## zoom in for gene expression
-            # observeEvent(input[[paste0("GeneExproup.clk", postfix)]],{
-            #     ranges$x <- NULL
-            #     ranges$y <- NULL
-            # })
             observeEvent(input[[paste0("GeneExproup.dbl", postfix)]],{
                 ranges$x <- NULL
                 ranges$y <- NULL
@@ -574,6 +573,10 @@ updateGeneExprDotPlotUI <-
                         value = '')
                 }
             })
+            observeEvent(input[[paste0("GeneExproup.clk", postfix)]],{
+                # print(input[[paste0("GeneExproup.clk", postfix)]])
+                
+            })
         }
         
         output[[paste0("GeneExproup.dwn", postfix)]] <-
@@ -1025,6 +1028,7 @@ updateSubsetGeneExprPlot <-
     }
 
 # sub module related
+#' @importFrom plotly toWebGL ggplotly renderPlotly event_register plotlyOutput
 updateSubModulePlotUI <-
     function(
         postfix = 1,
@@ -1033,20 +1037,48 @@ updateSubModulePlotUI <-
         input,
         output,
         session,
+        p_session,
         interactive,
         plotX,
         height,
+        lasso=FALSE,
         ...) {
         if (isTRUE(interactive)) {
             output[[paste0("GeneExproup", postfix)]] <-
                 renderPlotly({
-                    ggplotly(plotX()) %>% event_register("plotly_click")
+                    ggplotly(plotX()) %>% toWebGL() %>% event_register("plotly_click")
                 })
             output[[paste0("GeneExproup.ui", postfix)]] <- renderUI({
                 plotlyOutput(
                     NS0(NS(pid, id), "GeneExproup", postfix),
                     height = height)
             })
+            if(isTRUE(lasso)){
+                ## update the download form
+                updateSelectInput(session=session,
+                                  inputId = paste0("GeneExproup.fmt",postfix),
+                                  choices = 'CSV', selected = 'CSV')
+                updateCheckboxInput(session = session,
+                                    inputId = paste0('GeneExproupDimT', postfix),
+                                    value = FALSE)
+                output[[paste0("GeneExproup.dwn", postfix)]] <- 
+                    exprDownloadHandler(dataset=pid, lasso=TRUE,
+                                        plot=plotX, session=session)
+                observeEvent(input[[paste0('GeneExproupSelIDs', postfix)]], {
+                    d <- get_lasso_selected_ids(session=session, plot=plotX)
+                    if(length(d)){
+                        if(all(!is.na(d$sampleID))){
+                            p_session$userData$selectedCellIDs <- 
+                                as.character(d$sampleID)
+                            adminMsg('Succeed setting lasso selection',
+                                     type='message')
+                        }
+                    }
+                    updateActionButton(session = p_session,
+                                       inputId = 'filterCellIDs',
+                                       disabled = FALSE)
+                }, ignoreInit = TRUE)
+            }
         } else{
             output[[paste0("GeneExproup", postfix)]] <- renderPlot({
                 plotX()
@@ -1090,6 +1122,10 @@ subModuleMenuObservor <- function(
     observeEvent(input$resize, {
         updateTextInput(p_session, "resizePlotModule", value = id)
     })
+    observeEvent(input$CellInfoCoor1, {
+        updateTextInput(p_session, "changeCoorContext",
+                        value = paste(id, input$CellInfoCoor1, sep='___'))
+    })
     observeEvent(input$CellInfosubgrp1, {
         updateTextInput(p_session, "changeSubsetContext",
                         value = paste(id, input$CellInfosubgrp1, sep='___'))
@@ -1363,8 +1399,13 @@ filterCells <- function(
         valueFilterKey,
         valueFilterCutoff,
         valueFilterCutoff2,
-        inpConf) {
-    keep <- rep(TRUE, nrow(ggData))
+        inpConf,
+        lassoSelected) {
+    if(missing(lassoSelected)){
+        keep <- rep(TRUE, nrow(ggData))
+    }else{
+        keep <- lassoSelected
+    }
     if (!missing(subsetCellKey) && !missing(subsetCellVal)) {
         if(length(subsetCellKey)==1){
             if(!is.list(subsetCellVal)){
